package com.perforce.hws.testing.deployment.deb;

import com.perforce.hws.automation.Cleanup;
import com.perforce.hws.automation.DeploymentCleanup;
import com.perforce.hws.automation.InstallEnvironmentSetup;
import com.perforce.hws.automation.NetworkUtil;
import com.perforce.hws.util.TimingHelpers;
import com.perforce.hwsclient.ApiClient;
import com.perforce.hwsclient.ApiClient.HwsRestException;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.LoginRequest;
import org.testng.Assert;
import org.testng.annotations.Test;
import retrofit.RetrofitError;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

public class InvalidConfigTests implements DebDeployment, NetworkUtil, TimingHelpers {

    /**
     * When installing HWS and pointing it to a missing/illegal p4d, halt.
     */
    @Test
    public void invalidP4dBreaksStatus() throws Exception {
        DeploymentCleanup deploymentCleanup = new DeploymentCleanup();
        deploymentCleanup.run();

        String debBuild = obtainDebBuild();

        installDebFile(debBuild);

        // Add a file that points to ... nowhere
        createInvalidP4dConfig();

        // Now that we've reconfigured, we need to restart
        execOrFail("service", "helix-ws", "restart");

        boolean isRunning = waitFor(10, 1000, () -> !isPortAvailable(443));
        Assert.assertTrue(isRunning, "HWS isn't resstarted at port 443");

        ApiClient client = new ApiClient("https://localhost");

        Assert.assertFalse(client.isOK(), "HWS is not reporting an error status");
    }

    @Test
    public void incorrectCharsetBreaksStatus() throws Exception {
        DeploymentCleanup deploymentCleanup = new DeploymentCleanup();
        deploymentCleanup.run();

        try {
            String debBuild = obtainDebBuild();

            installDebFile(debBuild);

            boolean isRunning = waitFor(10, 1000, () -> !isPortAvailable(443));
            Assert.assertTrue(isRunning, "HWS isn't started at port 443");

            // This should detect that HWS is already running and do nothing, but will
            // configure a "localhost" p4d.
            InstallEnvironmentSetup setup = new InstallEnvironmentSetup();
            setup.run();

            createIncorrectCharsetP4dConfig();

            // Now that we've reconfigured, we need to restart
            execOrFail("service", "helix-ws", "restart");

            isRunning = waitFor(10, 1000, () -> !isPortAvailable(443));
            Assert.assertTrue(isRunning, "HWS isn't restarted at port 443");

            ApiClient client = new ApiClient(true, "https://localhost");

            Assert.assertFalse(client.isOK(), "HWS is not reporting an error status");

            RetrofitError error = null;
            try {
                client.setCredentials("jdoe", "johndoe1A!");
                client.createDefaultService().getDepots("invalid");
            } catch (HwsRestException re) {
                error = re.getRetrofitError();
            }

            Assert.assertNotNull(error);
            Assert.assertEquals(error.getResponse().getStatus(), 500);
        } finally {
            Cleanup cleanup = new Cleanup();
            cleanup.run();

            deploymentCleanup.run();
        }
    }

    private void createInvalidP4dConfig() throws IOException {
        StringBuilder sb = new StringBuilder();
        sb.append("id: invalid\n");
        sb.append("name: invalid\n");
        sb.append("description: Local p4d server that doesn't exist\n");
        sb.append("P4PORT: localhost:1234\n");
        sb.append("P4CHARSET: utf8\n");

        Path configPath = Paths.get("/", "opt", "perforce", "helix-web-services", "etc", "p4d", "invalid");

        Files.write(configPath, sb.toString().getBytes());
    }

    private void createIncorrectCharsetP4dConfig() throws IOException {
        StringBuilder sb = new StringBuilder();
        sb.append("id: localhost\n");
        sb.append("name: localhost\n");
        sb.append("description: Local p4d server that is usually unicode\n");
        sb.append("P4PORT: localhost:1234\n");
        sb.append("P4CHARSET: utf8\n");

        Path configPath = Paths.get("/", "opt", "perforce", "helix-web-services", "etc", "p4d", "localhost");

        Files.write(configPath, sb.toString().getBytes());

    }
}
