package com.perforce.hws.testing.deployment.rpm;

import com.perforce.hws.automation.OsHelpers;
import com.perforce.hws.automation.ProjectLocations;
import com.perforce.hws.testing.deployment.DeploymentConventions;
import com.perforce.hws.util.ProcessHelpers;
import com.perforce.hws.util.StreamReaderThread;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Created by tristan on 5/13/16.
 */
public interface RpmDeployment extends
        DeploymentConventions,
        OsHelpers,
        ProcessHelpers,
        ProjectLocations {

    Logger logger = LoggerFactory.getLogger(RpmDeployment.class);

    /**
     * This is the "original" RPM pacakge we use to test upgrades. It's an unreleased
     * old version of 2016.1 that has different package names.
     *
     * @param rpmFile
     */
    default void installOldRpmFile(String rpmFile) {
        try {
            logger.info("installing rpm package from {}", rpmFile);

            logger.info(execOutOrFail("yum", "install", "-y", rpmFile));
            logger.info(execOutOrFail("/opt/perforce/helix-web-services/sbin/configure-helix-web-services"));
            logger.info(execOutOrFail("service", "helix-ws", "start"));
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    default void installRpmFile(String rpmFile) {
        try {
            logger.info("installing rpm package from {}", rpmFile);

            logger.info(execOutOrFail("yum", "install", "-y", rpmFile));
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    default void upgradeRpmBuild() {
        try {
            String rpmFile = obtainRpmBuild();

            if (rpmFile == null) {
                throw new IllegalStateException("unable to locate .rpm package");
            }

            logger.info("upgrading HWS from {}", rpmFile);

            // This command fails, with "error code 2" and absolutely no error output on the EC
            // instance only.
            // Running the commands manually works.
            // We're going to start generating the ProcessBuilder call directly just to set
            // the working directory, etc.
//            logger.info(execOutOrFail("rpm", "-Uvh", rpmFile));

            ProcessBuilder pb = new ProcessBuilder();

//            pb.command("rpm", "-Uvh", Paths.get(rpmFile).getFileName().toString());
            pb.command("yum", "install", "-y", Paths.get(rpmFile).getFileName().toString());

            pb.directory(Paths.get(rpmFile).getParent().toFile());

            Process p = pb.start();

            StreamReaderThread stdout = new StreamReaderThread(p.getInputStream());
            stdout.start();

            StreamReaderThread stderr = new StreamReaderThread(p.getErrorStream());
            stderr.start();

            p.waitFor();

            stderr.join();
            stdout.join();

            logger.info("rpm -Uvh stderr: {}", stderr.getStringBuffer().toString());
            logger.info("rpm -Uvh stdout: {}", stdout.getStringBuffer().toString());

            if (p.exitValue() != 0) {
                throw new IllegalStateException("rpm -Uvh " +
                        Paths.get(rpmFile).getFileName().toString() + " failed");
            }

            logger.info(execOutOrFail("service", "helix-ws", "stop"));
            logger.info(execOutOrFail("service", "helix-ws", "start"));
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    default String obtainRpmBuild() {
        String pretestArchive = obtainRpmPretestArchive();
        logger.info("pretestArchive {}", pretestArchive);
        if (pretestArchive == null) {
            return localRpmBuildArchive();
        }
        return pretestArchive;
    }

    default String localRpmBuildArchive() {
        try {
            Path localArchive = Paths.get(projectRootOrFail(), "helix-web-services-rpm.tgz");
            if (Files.exists(localArchive)) {
                return unpackRpmArchive(localArchive.toAbsolutePath().toString());
            }
            return null;
        } catch (IOException io) {
            throw new UncheckedIOException(io);
        }
    }

    default String obtainRpmPretestArchive() {
        try {
            Path ecArchive = Paths.get(projectRootOrFail(), "..", "..", "p4-bin",
                    "bin.centos6x86_64", "helix-web-services-rpm-pretest.tgz");
            if (Files.exists(ecArchive)) {
                return unpackRpmArchive(ecArchive.toAbsolutePath().toString());
            }
            return null;
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    default String unpackRpmArchive(String installerArchive, String localDirName) {
        try {
            logger.info("unpacking {} to {}", installerArchive, localDirName);
            Files.createDirectories(Paths.get(localDirName));

            String tarOut;
            if (isVMWareFolder()) {
                tarOut = execOutOrFail("sudo", "-u", vmwareUser(), "tar", "-v", "-x", "-z", "-p", "-C", localDirName, "-f", installerArchive);
            } else {
                tarOut = execOutOrFail("tar", "-v", "-x", "-z", "-C", localDirName, "-f", installerArchive);
            }
            logger.info("tar: {}", tarOut);

            Path pkg = Files.newDirectoryStream(Paths.get(localDirName), "helix-web-services*.rpm").iterator().next();
            return pkg.toAbsolutePath().toString();

        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    default String unpackRpmArchive(String installerArchive) {
        return unpackRpmArchive(installerArchive, "tmp_rpm_installer_archive");
    }
}
