package com.perforce.hws.testing.reporting;

import com.esotericsoftware.yamlbeans.YamlReader;
import org.stringtemplate.v4.ST;
import org.stringtemplate.v4.STGroupFile;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Generates a single asciidoc document based on multiple result.yaml files.
 *
 * We basically read all yaml files from the input directory, because it's expected that our general
 * workflow will be to grab test results from multiple runs in EC.
 */
public class AsciidocReportGenerator {

    private String inputDir;
    private String outputFile;

    public AsciidocReportGenerator(String inputDir, String outputFile) {
        this.inputDir = inputDir;
        this.outputFile = outputFile;
    }

    public void generate() {
        try {
            List<Path> inputFiles = listInputFiles(inputDir);

            if (inputFiles.isEmpty()) {
                System.err.println("no input files found");
                return;
            }

            // Right now, let's just buffer everything into memory for simplicity's sake.
            List<MethodResult> results = readAllResults(inputFiles);

            // Collate the results by suite
            Map<String, List<MethodResult>> suiteResults =
                    results.stream()
                            .sorted((x, y) -> x.getName().compareTo(y.getName()))
                            .collect(Collectors.groupingBy(MethodResult::getSuite));

            // Generate summaries
            List<SuiteSummary> suiteSummaries = suiteResults.entrySet()
                    .stream()
                    .map((entry) -> {
                        SuiteSummary summary = new SuiteSummary(entry.getKey());
                        collateResults(summary, entry.getValue());
                        return summary;
                    })
                    .sorted((x, y) -> x.getName().compareTo(y.getName()))
                    .collect(Collectors.toList());

            AllSummary allSummary = new AllSummary();
            suiteSummaries.forEach(s -> {
                allSummary.addSuite();
                allSummary.addPassed(s.getPassed());
                allSummary.addFailed(s.getFailed());
                allSummary.addSkipped(s.getSkipped());
            });

            // Create our writer
            Path outputPath = Paths.get(outputFile);

            if (!Files.exists(outputPath.getParent())) {
                Files.createDirectories(outputPath.getParent());
            }

            try (FileWriter fileWriter = new FileWriter(outputFile)) {
                BufferedWriter writer = new BufferedWriter(fileWriter);

                STGroupFile groupFile = new STGroupFile("report.stg");

                ST summaryTemplate = groupFile.getInstanceOf("summary_report");
                summaryTemplate.add("summaries", suiteSummaries);
                summaryTemplate.add("allSummary", allSummary);

                writer.write(summaryTemplate.render());

                ST methodResultTemplate = groupFile.getInstanceOf("method_report");

                suiteResults.forEach((suiteName, methodResults) -> {
                    methodResultTemplate.add("suiteName", suiteName);
                    methodResultTemplate.add("methodResults", methodResults);

                    try {
                        writer.write(methodResultTemplate.render());
                    } catch (IOException e) {
                        throw new UncheckedIOException(e);
                    }

                    methodResultTemplate.remove("suiteName");
                    methodResultTemplate.remove("methodResults");
                });

                writer.flush();
            }


        } catch (IOException io) {
            throw new UncheckedIOException(io);
        }
    }

    private void collateResults(SuiteSummary summary, List<MethodResult> methodResults) {
        methodResults.forEach(r -> {
            switch (r.getResultType()) {
                case PASSED:
                    summary.addPassed();
                    break;

                case FAILED:
                    summary.addFailed();
                    break;

                case SKIPPED:
                    summary.addSkipped();
                    break;
            }
        });
    }

    private List<Path> listInputFiles(String inputDir) throws IOException {
        return Files.list(Paths.get(inputDir))
                .filter(p -> Files.isRegularFile(p))
                .filter(p -> p.getFileName().toString().endsWith(".yaml"))
                .collect(Collectors.toList());
    }

    private List<MethodResult> readAllResults(List<Path> inputFiles) {
        return inputFiles.stream()
                .flatMap(p -> readResults(p))
                .collect(Collectors.toList());
    }

    private Stream<MethodResult> readResults(Path path) {
        try {
            YamlReader yamlReader = new YamlReader(new FileReader(path.toFile()));
            yamlReader.getConfig().setClassTag("result", MethodResult.class);
            return ((List<MethodResult>)yamlReader.read(List.class, MethodResult.class)).stream();
        } catch (IOException io) {
            throw new UncheckedIOException(io);
        }
    }

    public static void main(String args[]) {
        try {
            new AsciidocReportGenerator(args[0], args[1]).generate();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }
}
