package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.AlphaApi;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.*;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.assertTrue;
import static org.testng.AssertJUnit.assertEquals;

/**
 * Tests for methods underneath /p4/:api/changes
 */
public class PerforceChangesTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canListChanges(DefaultApi api) throws IOException {
        List<ChangesCommand> changes = api.getChanges("localhost", null, null, null, null);
        assertTrue(changes.stream().anyMatch(m -> "1".equals(m.getChange())));
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canLimitByMax(DefaultApi api) throws IOException {
        List<ChangesCommand> changes = api.getChanges("localhost", 1, null, null, null);
        assertEquals(changes.size(), 1);
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canLimitByUser(DefaultApi api) throws IOException {
        List<ChangesCommand> changes = api.getChanges("localhost", null, null, "super", null);
        assertTrue(changes.stream().allMatch(m -> "super".equals(m.getUser())));
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canLimitByStatus(DefaultApi api) throws IOException {
        List<ChangesCommand> changes = api.getChanges("localhost", null, "submitted", null, null);
        assertTrue(changes.stream().allMatch(m -> "submitted".equals(m.getStatus())));
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canLimitByFiles(DefaultApi api) throws IOException {
        List<ChangesCommand> changes = api.getChanges("localhost", null, null, null, "//.../Experimental/README");
        assertTrue(changes.stream().anyMatch(m -> "1".equals(m.getChange())));
    }

    @Test(dataProvider = "ProjectClientAsJDoeWithAlpha",
            dataProviderClass = ClientProvider.class,
            groups = "no-cloud")
    public void canBranchFilesInClassicDepot(DefaultApi api, AlphaApi alphaApi) throws IOException {
        String rand = randomString();

        ChangelistRequest request = new ChangelistRequest();
        request.setDescription("testing integ");

        List<ChangelistAction> actions = new ArrayList<>();
        ChangelistAction action = new ChangelistAction();
        action.setDepotFile(format("//depot/test-%s/Experimental/README", rand));
        action.setFromDepotFile("//depot/dev/Experimental/README");
        action.setActionType("branch");
        actions.add(action);

        request.setActions(actions);

        CommandResponse response =
                alphaApi.addChange("localhost", request);

        assertTrue(
                response.getResults().stream().anyMatch(r -> r.containsKey("submittedChange")),
                "No submitted change returned in results"
        );
    }

    @Test(dataProvider = "ProjectClientAsJDoeWithAlpha",
            dataProviderClass = ClientProvider.class)
    public void canBranchFilesInStream(DefaultApi api, AlphaApi alphaApi) throws IOException {
        String rand = randomString();

        ChangelistRequest request = new ChangelistRequest();
        request.setDescription("testing stream integ");
        request.setStream("//my_project/main");

        List<ChangelistAction> actions = new ArrayList<>();
        ChangelistAction action = new ChangelistAction();
        action.setDepotFile(format("test-%s/Experimental/README", rand));
        action.setFromDepotFile("Experimental/README");
        action.setActionType("branch");
        actions.add(action);

        request.setActions(actions);

        CommandResponse response =
                alphaApi.addChange("localhost", request);

        assertTrue(
                response.getResults().stream().anyMatch(r -> r.containsKey("submittedChange")),
                "No submitted change returned in results"
        );
    }

    @Test(dataProvider = "ProjectClientAsJDoeWithAlpha",
            dataProviderClass = ClientProvider.class)
    public void canAddFilesToStream(DefaultApi api, AlphaApi alphaApi) throws IOException {
        String rand = randomString();

        ChangelistRequest request = new ChangelistRequest();
        request.setDescription("testing stream add");
        request.setStream("//my_project/main");

        List<ChangelistAction> actions = new ArrayList<>();
        ChangelistAction action = new ChangelistAction();
        action.setDepotFile(format("Experimental/teapot-%s", rand));
        action.setActionType("upload");
        action.setContent(Base64.getEncoder().encodeToString("I'm a little teapot".getBytes()));
        actions.add(action);

        request.setActions(actions);

        CommandResponse response =
                alphaApi.addChange("localhost", request);

        assertTrue(
                response.getResults().stream().anyMatch(r -> r.containsKey("submittedChange")),
                "No submitted change returned in results"
        );
    }

    @Test(dataProvider = "ProjectClientAsJDoeWithAlpha",
            dataProviderClass = ClientProvider.class,
            groups = "no-cloud")
    public void canAddFilesToClassicDepot(DefaultApi api, AlphaApi alphaApi) throws IOException {
        String rand = randomString();

        ChangelistRequest request = new ChangelistRequest();
        request.setDescription("testing classic depot add");

        List<ChangelistAction> actions = new ArrayList<>();
        ChangelistAction action = new ChangelistAction();
        action.setDepotFile(format("//depot/dev/Experimental/teapot-%s", rand));
        action.setActionType("upload");
        action.setContent(Base64.getEncoder().encodeToString("I'm a little teapot".getBytes()));
        actions.add(action);

        request.setActions(actions);

        CommandResponse response =
                alphaApi.addChange("localhost", request);

        assertTrue(
                response.getResults().stream().anyMatch(r -> r.containsKey("submittedChange")),
                "No submitted change returned in results"
        );
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canListChangeDetails(DefaultApi api) throws IOException {
        ChangeCommand changeSpec = api.getChange("localhost", "1");
        assertEquals("1", changeSpec.getChange());
    }
}
